/* GStreamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GST_RTP_H__
#define __GST_RTP_H__

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <fcntl.h>
#include <gst/gst.h>
#include <ortp.h>

/* We need to define our own payload types & specially for mpeg videos that
* wont look nice if included in files like rtp-audio.h */

typedef enum
{

/* Audio: */
  PAYLOAD_GSM = 3,
  PAYLOAD_L16_MONO = 11,
  PAYLOAD_MPA = 14,		/* Audio MPEG 1-3 */
  PAYLOAD_G723_63 = 16,		/* Not standard */
  PAYLOAD_G723_53 = 17,		/* Not standard */
  PAYLOAD_TS48 = 18,		/* Not standard */
  PAYLOAD_TS41 = 19,		/* Not standard */
  PAYLOAD_G728 = 20,		/* Not standard */
  PAYLOAD_G729 = 21,		/* Not standard */

/* Video: */
  PAYLOAD_MPV = 32,		/* Video MPEG 1 & 2 */

/* BOTH */
  PAYLOAD_BMPEG = 34		/* Not Standard */
}
rtp_payload_t;

typedef struct rtp_info rtp_sender_info;
typedef struct rtp_info rtp_receiver_info;

struct rtp_info
{
  guint16 seq;
  guint32 timestamp;
  guint32 packets_sent;
  guint32 octets_sent;
};

void rtp_lib_init();

void rtp_lib_close();

int find_host (gchar * host_name, struct in_addr *address);

void rtp_send (RtpSession * session, rtp_sender_info * sen, gpointer buf, int nbytes, guint32 nsamp);

gpointer rtp_receive (RtpSession * session, rtp_receiver_info * recv, guint32 mtu, guint32 nsamp);

int rtp_connection_call (RtpSession * session, rtp_sender_info *sen, gchar * remote, guint port);
int rtp_server_init (RtpSession * session, rtp_receiver_info *recv, gchar * local, guint port);

GstCaps *mediatype_to_caps (const gchar *media_type, rtp_payload_t *pt, 
					     	guint *mtu );

const gchar *caps_to_mediatype (GstCaps *);

#endif
